/// <reference types="socket.io-client" />
import Promise from 'bluebird';
/**
 * This event indicates a JSON-RPC request which expects a response.
 */
export declare const RPCEvent = "rpc";
/**
 * RPCRequest represents an RPC request which should be sent to the socket
 * identified by the segment of the method before the ".".
 *
 * @export
 * @interface RPCRequest
 */
export interface RPCRequest {
    /**
     * A unique ID used to correlate the response message.
     *
     * @type {string}
     * @memberof RPCRequest
     */
    id: string;
    /**
     * A sender should set the method using the format `{socketID}.{handlerName}.{methodName}`.
     * The Live service will propegate the request to `{socketID}` with the method changed to `{handlerName}.{methodName}`.
     *
     * @type {string}
     * @memberof RPCRequest
     */
    method: string;
    params: any[];
}
/**
 * Events emitted by socket.io.
 */
export declare type SocketIOEvent = "connect" | "connect_error" | "connect_timeout" | "error" | "disconnect" | "reconnect" | "reconnect_attempt" | "reconnecting" | "reconnect_error" | "reconnect_failed";
/**
 * RPCResponse contains the response to an RPC request.
 *
 * @export
 * @interface RPCResponse
 */
export interface RPCResponse {
    id: any;
    /**
     * The result of the RPC request. Must be null if there is an error.
     *
     * @type {*}
     * @memberof RPCResponse
     */
    result?: any;
    error?: any;
}
export interface RPCError {
    code: number;
    message: string;
    data?: any;
}
export declare class LiveClient {
    private socket;
    private id;
    constructor(socket: SocketIOClient.Socket);
    /**
     * The ID of the socket.io socket this client is using.
     *
     * @readonly
     * @type {string}
     * @memberof LiveClient
     */
    readonly socketID: string;
    /**
     * Authenticate authenticates this client with the Live service. Nothing else will work until you call this.
     * The returned promise will resolve with `true` if you are authenticated.
     *
     * @param {string} token
     * @returns {Promise<boolean>}
     * @memberof LiveClient
     */
    authenticate(token: string): Promise<boolean>;
    /**
     * Disconnects from Live service.
     *
     * @memberof LiveClient
     */
    disconnect(): void;
    /**
     * Sends a notification, which expects no reply.
     *
     * @template TRequest
     * @param {string} method
     * @param {TRequest} param
     * @param {(error: shared.RPCError, result: TResponse) => void} callback
     * @memberof LiveClient
     */
    notify<TRequest>(method: string, param: TRequest): void;
    /**
     * Calls a method over RPC. Returns a promise which will be resolved with the result of the call, or rejected if there is an error.
     *
     * @template TResponse
     * @param {string} address - The address to send the request to. This will be the ID of the socket of the component you want to send the request to.
     * @param {string} method - The method to invoke.
     * @param {TRequest} param - The parameter object of the method.
     * @returns {Promise<TResponse>}
     * @memberof LiveClient
     */
    request<TResponse>(address: string, method: string, param: any): Promise<TResponse>;
    /**
     * Wires up a handler for the socket.io events. The primary use of this is for `client.on('connect', () => { /* you're connected!*\/  })`
     *
     * @param {SocketIOEvent} event
     * @param {Function} callback
     * @memberof LiveClient
     */
    on(event: SocketIOEvent, callback: Function): void;
    /**
     * Wires up a handler for a normal socket.io notification with no response/ack.
     *
     * @template TNotification
     * @param {string} method
     * @param {(notification: TNotification) => void} callback
     * @memberof LiveClient
     */
    onNotification<TNotification>(method: string, callback: (notification: TNotification) => void): void;
    /**
     * Wires up a handler for an RPC-style request.
     *
     * @template TRequest
     * @template TResponse
     * @param {string} method - The method this handler will listen for.
     * @param {(request: TRequest) => Promise<TResponse>} handler - The handler for the request. It must return a promise containing the response.
     * @memberof LiveClient
     */
    onRequest<TRequest, TResponse>(method: string, handler: (request: TRequest) => Promise<TResponse>): void;
}
/**
 * Client for the Pipeline namespace.
 *
 * @export
 * @class PipelineLiveClient
 * @extends {LiveClient}
 */
export declare class PipelineLiveClient extends LiveClient {
    constructor(socket: SocketIOClient.Socket);
    /**
     * Gets the registered agents, as a hash of agentID:socketID.
     *
     * @returns {Promise<{[agentID:string]:string}>}
     * @memberof PipelineLiveClient
     */
    getRegisteredAgents(): Promise<{
        [agentID: string]: string;
    }>;
}
