"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var bluebird_1 = require("bluebird");
/**
 * This event indicates a JSON-RPC request which expects a response.
 */
exports.RPCEvent = "rpc";
var LiveClient = /** @class */ (function () {
    function LiveClient(socket) {
        this.socket = socket;
        this.id = 0;
    }
    Object.defineProperty(LiveClient.prototype, "socketID", {
        /**
         * The ID of the socket.io socket this client is using.
         *
         * @readonly
         * @type {string}
         * @memberof LiveClient
         */
        get: function () {
            return this.socket.id;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Authenticate authenticates this client with the Live service. Nothing else will work until you call this.
     * The returned promise will resolve with `true` if you are authenticated.
     *
     * @param {string} token
     * @returns {Promise<boolean>}
     * @memberof LiveClient
     */
    LiveClient.prototype.authenticate = function (token) {
        return this.request("", "Live.Authenticate", token);
    };
    /**
     * Disconnects from Live service.
     *
     * @memberof LiveClient
     */
    LiveClient.prototype.disconnect = function () {
        this.socket.disconnect();
    };
    /**
     * Sends a notification, which expects no reply.
     *
     * @template TRequest
     * @param {string} method
     * @param {TRequest} param
     * @param {(error: shared.RPCError, result: TResponse) => void} callback
     * @memberof LiveClient
     */
    LiveClient.prototype.notify = function (method, param) {
        this.socket.emit(method, param);
    };
    /**
     * Calls a method over RPC. Returns a promise which will be resolved with the result of the call, or rejected if there is an error.
     *
     * @template TResponse
     * @param {string} address - The address to send the request to. This will be the ID of the socket of the component you want to send the request to.
     * @param {string} method - The method to invoke.
     * @param {TRequest} param - The parameter object of the method.
     * @returns {Promise<TResponse>}
     * @memberof LiveClient
     */
    LiveClient.prototype.request = function (address, method, param) {
        var _this = this;
        this.id++;
        return new bluebird_1.default(function (resolve, reject) {
            _this.socket.emit(exports.RPCEvent, { id: _this.id.toString(), method: address + "." + method, params: [param] }, function (response) {
                if (response.error) {
                    return reject(response.error);
                }
                resolve(response.result);
            });
        });
    };
    /**
     * Wires up a handler for the socket.io events. The primary use of this is for `client.on('connect', () => { /* you're connected!*\/  })`
     *
     * @param {SocketIOEvent} event
     * @param {Function} callback
     * @memberof LiveClient
     */
    LiveClient.prototype.on = function (event, callback) {
        this.socket.on(event, callback);
    };
    /**
     * Wires up a handler for a normal socket.io notification with no response/ack.
     *
     * @template TNotification
     * @param {string} method
     * @param {(notification: TNotification) => void} callback
     * @memberof LiveClient
     */
    LiveClient.prototype.onNotification = function (method, callback) {
        this.socket.on(method, callback);
    };
    /**
     * Wires up a handler for an RPC-style request.
     *
     * @template TRequest
     * @template TResponse
     * @param {string} method - The method this handler will listen for.
     * @param {(request: TRequest) => Promise<TResponse>} handler - The handler for the request. It must return a promise containing the response.
     * @memberof LiveClient
     */
    LiveClient.prototype.onRequest = function (method, handler) {
        this.socket.on(exports.RPCEvent, function (request, callback) {
            if (request.method === method) {
                handler(request.params[0]).catch(function (err) { return callback({ id: request.id, error: { code: 500, message: err } }); })
                    .then(function (result) { return callback({ id: request.id, result: result }); });
            }
        });
    };
    return LiveClient;
}());
exports.LiveClient = LiveClient;
/**
 * Client for the Pipeline namespace.
 *
 * @export
 * @class PipelineLiveClient
 * @extends {LiveClient}
 */
var PipelineLiveClient = /** @class */ (function (_super) {
    __extends(PipelineLiveClient, _super);
    function PipelineLiveClient(socket) {
        return _super.call(this, socket) || this;
    }
    /**
     * Gets the registered agents, as a hash of agentID:socketID.
     *
     * @returns {Promise<{[agentID:string]:string}>}
     * @memberof PipelineLiveClient
     */
    PipelineLiveClient.prototype.getRegisteredAgents = function () {
        return this.request("Pipeline", "GetRegisteredAgents", {});
    };
    return PipelineLiveClient;
}(LiveClient));
exports.PipelineLiveClient = PipelineLiveClient;
